# -*- coding: utf-8 -*-
from afpy.ldap.connection import Connection as BaseConnection
from afpy.ldap.node import Node
from afpy.ldap.node import User as BaseUser
from afpy.ldap.node import GroupOfNames as BaseGroup
from afpy.ldap.utils import to_string, to_python
from afpy.ldap import schema
from IPython.Shell import IPShellEmbed
from optparse import OptionParser
import ldap
import sys

class Nodes(dict):
    def update(self, nodes):
        uids = self.keys()
        for uid in uids:
            del self[uid]
            del self.__dict__[uid]
        for n in nodes:
            uid = getattr(n, 'uid', n.cn) or n.cn or ''
            uid = uid.lower()
            for c in ' -_.':
                uid = uid.replace(c, '_')
            self[uid] = n
            self.__dict__[uid] = n
    def __getattr__(self, attr):
        return self.get(attr)
    def __repr__(self):
        return repr(sorted(self.keys()))
    def __delattr__(self, uid):
        n = self[uid]
        try:
            conn.delete(n)
        except Exception, e:
            print e
        else:
            del self[uid]
            del self.__dict__[uid]
    def __str__(self):
        out = []
        for k in sorted(self.keys()):
            out.append(str(self[k]))
            out.append('')
        return '\n'.join(out)

users = Nodes()
groups = Nodes()

class User(BaseUser):
    """
    >>> print users.gpasgrimaud
    >>> users.gpasgrimaud.mail = 'gael.pasgrimaud@alterway.fr'
    >>> users.gpasgrimaud.save()
    """

    _rdn = 'cn'
    _base_dn = 'ou=Users,dc=alterway,dc=fr'
    _defaults = dict(
        objectClass = [
                  'top',
                  'person',
                  'organizationalPerson',
                  'inetOrgPerson',
                  'posixAccount',
                  ],
        loginShell = '/bin/bash',
       )

    uid=schema.StringProperty('uid', title='uid', required=True)
    uidNumber=schema.IntegerProperty('uidNumber', title='uidNumber', required=True)
    gidNumber=schema.IntegerProperty('gidNumber', title='gidNumber', required=True)
    homeDirectory=schema.StringProperty('homeDirectory', title='homeDirectory', required=True)
    cn=schema.StringProperty('cn', title='cn', required=True)
    title=schema.UnicodeProperty('title', title='Title', required=True)
    sn=schema.UnicodeProperty('sn', title='sn', required=True)
    displayName=schema.UnicodeProperty('displayName', title='Nom affiche', required=True)
    givenName=schema.UnicodeProperty('givenName', title='Nom', required=True)
    mail=schema.StringProperty('mail', title='E-mail', required=True)
    telephoneNumber=schema.StringProperty('telephoneNumber', title='Tel.', required=True)
    mobile=schema.StringProperty('mobile', title='Port.', required=True)
    postalAddress=schema.UnicodeProperty('postalAddress', title='Adresse', required=True)
    postalCode=schema.StringProperty('postalCode', title='CP', required=True)
    st=schema.StringProperty('st', title='Pays', required=True)

    def update(self):
        p, n = self.cn.split('.', 1)
        if not self.uid:
            initials = ''.join([i[0] for i in p.split('-')])
            self.uid = '%s%s' % (initials, n)
        if not self.mail:
            self.mail = '%s@alterway.fr' % self.cn
        if not self.sn:
            self.sn = n.title()
        if not self.givenName:
            displayName = '-'.join([i.title() for i in p.split('-')])
            self.givenName = p.title()
        if not self.displayName:
            self.displayName = '%s %s' % (self.givenName, self.sn)
        if not self.homeDirectory:
            self.homeDirectory = '/home/%s' % self.uid
        uid = next_uid()
        if not self.uidNumber:
            self.uidNumber = uid
        if not self.gidNumber:
            self.gidNumber = self.uidNumber

    def save(self):
        try:
            BaseUser.save(self)
        except ldap.NO_SUCH_OBJECT:
            self._conn.add(self)

    def __str__(self):
        out = []
        def g(name):
            return getattr(self, name, None)
        title = '%s (%s)' % (self.displayName, self.cn)
        out.append(title)
        out.append("-"*len(title))
        out.append('%-15.15s: %s' % ('dn', self.dn))
        for name in ('uid', 'cn', 'sn', 'givenName', 'title', 'mobile', 'mail', 'uidNumber', 'gidNumber', 'homeDirectory'):
            v = g(name)
            if v:
                if isinstance(v, basestring):
                    v = v.replace('$', '\n         ')
                out.append('%-15.15s: %s' % (name, v))
        value = '\n'.join([o.encode('ascii', 'replace') for o in out if o])
        return value

class Group(BaseGroup):
    """
    >>> print groups
    >>> print groups.group1
    >>> groups.group1.add_users(users.gpasgrimaud)
    >>> groups.group1.save()
    """
    _rdn = 'cn'
    _base_dn = 'cn=Business Unit,dc=alterway,dc=fr'
    _defaults = dict(
        objectClass = [
                  'top',
                  'groupOfNames',
                  ],
       )

    @property
    def member(self):
        members = self._data['member']
        if not members:
            members = []
        if not isinstance(members, list):
            members = [members]
        return sorted(members)

    def add_users(self, *users):
        members = self.member
        for u in users:
            assert isinstance(u, User), 'param must be a User'
            members.append(u.dn)
        self._data['member'] = list(set(members))
        return self.member

    def del_users(self, *users):
        members = self.member
        for u in users:
            assert isinstance(u, User), 'param must be a User'
        dns = [u.dn for u in users]
        members = [dn for dn in members if dn not in dns]
        self._data['member'] = list(set(members))
        return self.member

    def __str__(self):
        out = []
        out.append('%s' % self.cn)
        out.append('-'*len(self.cn))
        out.append(', '.join([m.split(',', 1)[0].split('=')[1] for m in self.member]))
        value = '\n'.join(out)
        return value

    def __repr__(self):
        return '<Group at %s (%s)>' % (self.cn, len(self.member))

def get_conn(section='alterway'):
    """return a ldap connection"""
    class Connection(BaseConnection):
        """Specific connection"""
        user_class = User
    return Connection(section=section)

conn = get_conn()

def search(self, name, verbose=True):
    if name and name not in '*-':
        filter = '(|(uid=*%(name)s*)(cn=*%(name)s*))' % dict(name=name)
    else:
        filter = 'cn=*'
    nodes = conn.search_nodes(node_class=User, base_dn=User._base_dn, filter=filter)
    nodes.sort(cmp=lambda a, b: cmp(a.cn, b.cn))
    u = None
    if len(nodes) == 1:
        u = nodes[0]
    users.update(nodes)
    print users
    if verbose:
        print_vars(u)

def next_uid():
    uid = max([int(v.uidNumber) for v in conn.search_nodes(filter='uidNumber=*', attrs=['uidNumber'])])
    return uid+1

def new_user(self, cn=''):
    if not cn:
        cn = raw_input('CN: ')
    u = User()
    u.cn = cn
    u._update_dn(cn)
    u.update()
    u._conn = conn
    users.update([u])
    print u
    print_vars(u)

def save(self, *args, **kwargs):
    for nodes in (users, groups):
        for n in nodes.values():
            print 'Saving', n.cn
            try:
                n.save()
            except Exception, e:
                print '%s' % e

def print_vars(u=None):
    print ''
    print 'Session vars'
    print '------------'
    print 'conn = %r' % conn
    print 'users = %r' % users
    print 'groups = %r' % groups
    if u:
        print 'u =', repr(u)
        __IPYTHON__.api.to_user_ns('u')

def main():
    parser = OptionParser()
    parser.usage = """%%prog [-i] [-g pattern]

Config file: $HOME/.ldap.cfg

Examples:

    >>> search gpas #search for pattern gpas and put results in users var
    Gael Pasgrimaud (gael.pasgrimaud)
    ---------------------------------
    dn             : cn=gael.pasgrimaud,ou=Users,dc=alterway,dc=fr
    ...
    
    >>> save #save all users/groups loaded

    >>> new_user bruno.durand
    Bruno Durand (bruno.durand)
    ---------------------------
    dn             : cn=bruno.durand,ou=Users,dc=alterway,dc=fr
    ...
    >>> u.save()

%s
%s""" % (User.__doc__, Group.__doc__)
    parser.add_option("-g", "--grep", dest="grep",
                      default=None, metavar='PATTERN',
                      help="search for PATTERN")
    parser.add_option("-i", "--interactive", dest="interactive",
                      action="count", default=0,
                      help='Start an ipython session')
    parser.add_option("-v", "--verbose", dest="verbose",
                      action="count", default=0)
    options, args = parser.parse_args()

    u = None

    if not options.grep and not options.interactive:
        parser.parse_args(['-h'])

    if options.grep:
        search(None, options.grep, False)
        if users:
            u = users[sorted(users.keys())[0]]

    if options.interactive:
        sys.argv = sys.argv[0:1]
        import IPython.ipapi
        ip = IPython.ipapi.get()
        ipshell = IPShellEmbed()
        __IPYTHON__.api.expose_magic('new_user', new_user)
        __IPYTHON__.api.expose_magic('search', search)
        __IPYTHON__.api.expose_magic('save', save)
        results = conn.search_nodes(node_class=Group, base_dn=Group._base_dn, filter='objectClass=groupOfNames')
        results.sort(cmp=lambda a, b: cmp(a.cn, b.cn))
        groups.update(results)
        print_vars(u)

        ipshell()
